#!/usr/bin/env python
#
""" Give here the function description(s) """

import string
import re

underscore = re.compile(r'(?P<lead>.*?[^\\])(?P<text>[_#$&])(?P<trail>.*)')

def escape_underscore( string ):
    match = underscore.match( string )
    while match:
        string = r'%(lead)s\%(text)s%(trail)s' % match.groupdict() 
        match = underscore.match( string )
    return string

def translate( fp ):
    read_next = True
    firstlevel = True
    while 1:
        if read_next:
            line = fp.readline()
            if not line: break
        else:
            read_next = True
            
        words = line.split()
        if len(words) < 1: continue

        if words[0] == '.IDENTifer':
            cbuff = escape_underscore( words[1] )
            if firstlevel:
                header = '\subsection{%s}' % cbuff
                firstlevel = False
            else:
                header = '\n\\subsubsection{%s}' % cbuff
            header += '\n\\begin{tabular}{ll}'
            header += '\nIdentifier & %s \\\\' % cbuff
        elif words[0] == '.AUTHOR':
            cbuff = '\nAuthor &'
            for word in words[1:]: cbuff += ' %s' %  word
            header += '%s\\\\' % cbuff
        elif words[0] == '.LANGUAGE':
            cbuff = ''
            for word in words[1:]: cbuff += '%s ' %  word
            header += '\nLanguage & %s\\\\' % cbuff
        elif words[0] == '.PURPOSE':
            cbuff = '\nPurpose &'
            for word in words[1:]:
                cbuff += ' %s' %  escape_underscore( word )
            header += '%s\\\\' % cbuff
            while 1:
                line = fp.readline()
                if line[0] != '.' and line[0:4] != '-----':
                    header += '\n & %s\\\\' % line[0:-1].strip()
                else:
                    read_next = False
                    break
        elif words[0] == '.INPUT/OUTPUT':
            print_tt = False
            while 1:
                line = fp.readline()
                if line[0] == '.':
                    read_next = False
                    break                    
                elif len(line.split()) == 0:
                    continue
                else:
                    words = line.split()
                    if words[0] == 'call':
                        print_tt = True
                        cbuff = '\\texttt{'
                        for word in words[2:]: cbuff += '%s ' %  word
                        cbuff = 'Usage & ' + escape_underscore(cbuff) + '}'
                    elif words[0] == 'input:':
                        print_tt = False
                        cbuff = '\\hspace*{4ex}Input & '
                        line = fp.readline()
                        cbuff += escape_underscore(line[0:-1].strip())
                        words = line.split()
                        if words[0] == 'struct':
                            cbuff + ' (\\ref{struct:%s})' % words[1]
                    elif words[0] == 'in/output:':
                        print_tt = False
                        cbuff = 'In/Output & '
                        line = fp.readline()
                        cbuff += escape_underscore(line[0:-1].strip())
                        words = line.split()
                        if words[0] == 'struct':
                            cbuff += ' (\\ref{struct:%s})' % words[1]
                    elif words[0] == 'output:':
                        print_tt = False
                        cbuff = '\\hspace*{3ex}Output & '
                        line = fp.readline()
                        cbuff += escape_underscore(line[0:-1].strip())
                        words = line.split()
                        if words[0] == 'struct':
                            cbuff += ' (\\ref{struct:%s})' % words[1]
                    else:
                        if print_tt:
                            cbuff = ' & \\texttt{\\hspace*{8ex}'
                        else:
                            cbuff = ' & '
                        cbuff += escape_underscore(' ' + line[0:-1].strip())
                        words = line.split()
                        if words[0] == 'struct':
                            cbuff += ' (\\ref{struct:%s})' % words[1]
                        if print_tt: cbuff += '}'
                    header += '\n%s\\\\' % cbuff
        elif words[0] == '.RETURNS':
            cbuff = '\nReturns &'
            for word in words[1:]:
                cbuff += ' %s' %  escape_underscore(word)
            header += '%s\\\\' % cbuff
            while 1:
                line = fp.readline()
                if line[0] != '.':
                    header += '\n & %s\\\\' % \
                              escape_underscore(line[0:-1].strip())
                else:
                    read_next = False
                    break
        elif words[0] == '.COMMENTS':
            write_header = True
            cbuff = '\nComment &'
            for word in words[1:]:
                if word == 'static':
                    write_header = False
                    continue
                cbuff += ' %s' % escape_underscore(word)
            header += '%s\\\\' % cbuff
            while 1:
                line = fp.readline()
                if line[0] != '.' and line[0:4] != '----':
                    header += '\n & %s\\\\' % \
                              escape_underscore(line[0:-1].strip())
                else:
                    read_next = False
                    break
            header += '\n\\end{tabular}'
            if write_header: print header

if __name__ == '__main__':
    import glob
    from optparse import OptionParser

    usage = "usage: %prog [options]"
    parser = OptionParser(usage)
    parser.add_option("--directory", dest="directory", type='string')
    parser.set_defaults( directory='./' )
    (options, args) = parser.parse_args()

    files = glob.glob( args[0] + '/*.c' )
    files.sort()
    for flname in files:
        print '%% %s' % flname
        fp = open( flname, 'r' )
        translate( fp )
        fp.close()
