/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 1999 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   DEFS_NADC
.AUTHOR      R.M. van Hees 
.KEYWORDS    header file
.LANGUAGE    ANSI-C
.PURPOSE     definitions for error handling and macro's
.COMMENTS    None
.ENVIRONment None
.VERSION      3.8   09-Apr-2012	removed definition SCIA_SDMF_0, RvH
              3.7   23-Aug-2006	added calibration options SCIA_SDMF_0, RvH
              3.6   16-Aug-2006	added instrument equals SCIA_SDMF_0, RvH
              3.5   25-Aug-2003	added instrument equals SCIA_PATCH_1, RvH
              3.4   08-Apr-2003	added definition of NaN, RvH
              3.3   31-Mar-2003	modified include for C++ code
              3.2   08-Nov-2002	new calibration options for Sciamachy, RvH 
              3.1   12-Apr-2002	added write_info to param structure, RvH
              3.0   15-Nov-2001	one and only include file(!), RvH 
              2.4   29-Aug-2001 moved architecture dependent stuff 
                                to defs_arch.h, RvH
              2.3   29-Aug-2001 reorganisation & added documentation, RvH
              2.2   23-Aug-2001 replaced MYMIN/MYMAX with Type_MIN/Type_MAX,
                                RvH
              2.1   27-Jul-2001 added calibration options
                                calibration flag from char to short, RvH
              2.0   13-May-2001 split up library, RvH
              1.0   20-May-1999 creation by R.M. van Hees
------------------------------------------------------------*/
#ifndef  __DEFS_NADC	                        /* Avoid redefinitions */
#define  __DEFS_NADC

#ifdef __cplusplus
extern "C" {
#endif

/*+++++ Additional include files +++++*/
#include <stdbool.h>
#include <float.h>
#include <defs_arch.h>

/*+++++ General Macros +++++*/
#ifndef TRUE
#define TRUE		true
#endif
#ifndef FALSE
#define FALSE		false
#endif

#ifdef __GNUC__
#define min(x,y) ({ \
        const typeof(x) _x = (x);       \
        const typeof(y) _y = (y);       \
        (void) (&_x == &_y);            \
        _x < _y ? _x : _y; })
#define max(x,y) ({ \
        const typeof(x) _x = (x);       \
        const typeof(y) _y = (y);       \
        (void) (&_x == &_y);            \
        _x > _y ? _x : _y; })
#define min_t(type,x,y) \
        ({ type __x = (x); type __y = (y); __x < __y ? __x: __y; })
#define max_t(type,x,y) \
        ({ type __x = (x); type __y = (y); __x > __y ? __x: __y; })
#else
#define min_t(type,x,y) \
        ((type)(x) < (type)(y) ? (type)(x): (type)(y))
#define max_t(type,x,y) \
        ((type)(x) > (type)(y) ? (type)(x): (type)(y))
#endif

/*+++++ some useful constants +++++*/
#ifndef PI
#ifndef M_PI
#define PI		3.14159265358979323846
#else
#define PI		M_PI
#endif
#endif
#define DEG2RAD		(PI / 180.)

#define R_EARTH		6378.137     /* equatorial radius of the Earth (km) */

#define CHAR_ZERO    ((char) 0)
#define SCHAR_ZERO   ((signed char) 0)
#define UCHAR_ZERO   ((unsigned char) 0)
#define UCHAR_ONE    ((unsigned char) 1)
#define USHRT_ZERO   ((unsigned short) 0)
#define UINT_ZERO    ((unsigned int) 0)

#define NUM_CORNERS     4

/* constants used by nadc_akima.c */
#define AKIMA_EXTRA_POINTS         4

/* constants used by nadc_init_param.c */
#define PARAM_SET    ((unsigned char) TRUE)
#define PARAM_UNSET  ((unsigned char) FALSE)

#define SHORT_STRING_LENGTH        ((size_t) 80)
#define MAX_STRING_LENGTH          ((size_t) 256)
#define GOME_DATE_LENGTH           25
#define SCIA_DATE_LENGTH           28
#define DATE_ONLY_STRING_LENGTH    11
#define TIME_ONLY_STRING_LENGTH    12

/* filename sizes for different platforms */
#define ENVI_FILENAME_SIZE         63
#define ERS2_FILENAME_SIZE         67

/*
 * general PDS definitions (not data level dependent)
 */
#define PDS_MPH_LENGTH        1247
#define PDS_DSD_LENGTH         280

#define PDS_ASCII_HDR_LENGTH   (PDS_DSD_LENGTH+2)
#define PDS_KEYWORD_LENGTH      41
#define PDS_KEYVAL_LENGTH      192

#define ENVI_CHAR   ((size_t) 1)
#define ENVI_UCHAR  ((size_t) 1)
#define ENVI_SHORT  ((size_t) 2)
#define ENVI_USHRT  ((size_t) 2)
#define ENVI_INT    ((size_t) 4)
#define ENVI_UINT   ((size_t) 4)
#define ENVI_LLONG  ((size_t) 8)
#define ENVI_FLOAT  ((size_t) 4)
#define ENVI_DBLE   ((size_t) 8)

/* length of the Scia Date string in the PDS ASCII headers */
#define MAX_UTC_STRING          28

/* flags for GOME and SCIAMACHY spectral band selection */
#define BAND_NONE     ((unsigned char) 0x0U)
#define BAND_ONE      ((unsigned char) 0x1U)
#define BAND_TWO      ((unsigned char) 0x2U)
#define BAND_THREE    ((unsigned char) 0x4U)
#define BAND_FOUR     ((unsigned char) 0x8U)
#define BAND_FIVE     ((unsigned char) 0x10U)
#define BAND_SIX      ((unsigned char) 0x20U)
#define BAND_SEVEN    ((unsigned char) 0x40U)
#define BAND_EIGHT    ((unsigned char) 0x80U)
#define BAND_ALL      (BAND_ONE|BAND_TWO|BAND_THREE|BAND_FOUR|\
                       BAND_FIVE|BAND_SIX|BAND_SEVEN|BAND_EIGHT)

/* definition of the GOME sub-channels */
#define BAND_ONE_A    BAND_ONE
#define BAND_TWO_A    BAND_TWO
#define BAND_ONE_B    BAND_SEVEN
#define BAND_TWO_B    BAND_EIGHT

/* flags for ground pixel selection */
#define SUBSET_NONE   ((unsigned char) 0x0U)
#define SUBSET_EAST   ((unsigned char) 0x1U)
#define SUBSET_CENTER ((unsigned char) 0x2U)
#define SUBSET_WEST   ((unsigned char) 0x4U)
#define SUBSET_BACK   ((unsigned char) 0x8U)
#define SUBSET_ALL    (SUBSET_EAST|SUBSET_CENTER|SUBSET_WEST|SUBSET_BACK)

/* flags for calibration selection */
#define CALIB_NONE       (0x0U)

#define GOME_CAL_LEAK    ((unsigned short) 0x1U)
#define GOME_CAL_FPA     ((unsigned short) 0x2U)/* only Earth data (band 1a) */
#define GOME_CAL_BSDF    ((unsigned short) 0x2U)          /* only Solar data */
#define GOME_CAL_FIXED   ((unsigned short) 0x4U)
#define GOME_CAL_STRAY   ((unsigned short) 0x8U)
#define GOME_CAL_NORM    ((unsigned short) 0x10U)
#define GOME_CAL_ALBEDO  ((unsigned short) 0x20U)
#define GOME_CAL_POLAR   ((unsigned short) 0x40U)
#define GOME_CAL_INTENS  ((unsigned short) 0x80U)
#define GOME_CAL_JUMPS   ((unsigned short) 0x100U)
#define GOME_CAL_AGING   ((unsigned short) 0x200U)
#define GOME_CAL_UNIT    ((unsigned short) 0x400U)

#define GOME_CAL_PMD     (GOME_CAL_LEAK|GOME_CAL_POLAR|GOME_CAL_INTENS)
#define GOME_CAL_EARTH   (GOME_CAL_LEAK|GOME_CAL_FPA|GOME_CAL_FIXED|\
                          GOME_CAL_STRAY|GOME_CAL_NORM|GOME_CAL_POLAR|\
                          GOME_CAL_INTENS)
#define GOME_CAL_MOON    (GOME_CAL_LEAK|GOME_CAL_FPA|GOME_CAL_FIXED|\
                          GOME_CAL_STRAY|GOME_CAL_NORM|GOME_CAL_INTENS)
#define GOME_CAL_SUN     (GOME_CAL_LEAK|GOME_CAL_FIXED|GOME_CAL_STRAY|\
                          GOME_CAL_NORM|GOME_CAL_BSDF|GOME_CAL_INTENS)

  /* Scia Lv0 calibration options (avoid conflicts with Lv1b options!) */
#define DO_CORR_COADDF   (0x10000000U)
#define DO_CORR_NORM     (0x20000000U)

  /* Scia Lv1b calibration options */
#define DO_CORR_VIS_MEM  (0x1U)
#define DO_SRON_MEM_NLIN (0x2U)
#define DO_CORR_IR_NLIN  (0x4U)
#define DO_SRON_DARK     (0x8U)
#define DO_CORR_AO       (0x10U)
#define DO_CORR_DARK     (0x20U)
#define DO_CORR_VDARK    (0x40U)
#define DO_CORR_VSTRAY   (0x80U)
#define DO_CORR_ADARK    (0x100U)
#define DO_CORR_LDARK    (0x200U)
#define DO_SRON_PPG      (0x400U)
#define DO_CORR_PPG      (0x800U)
#define DO_FIXED_PPG     (0x20000U)
#define DO_CORR_ETALON   (0x1000U)
#define DO_SRON_STRAY    (0x2000U)
#define DO_CORR_STRAY    (0x4000U)
#define DO_SRON_WAVE     (0x8000U)
#define DO_CALIB_WAVE    (0x10000U)
#define DO_CORR_POL      (0x40000U)
#define DO_SRON_RAD      (0x80000U)
#define DO_CORR_RAD      (0x100000U)
#define DO_SRON_BDPM     (0x200000U)
#define DO_MASK_BDPM     (0x400000U)
#define DO_DIVIDE_SUN    (0x800000U)
#define DO_SRON_SUN      (0x1000000U)
#define DO_CALC_ERROR    (0x2000000U)
#define DO_SRON_NOISE    (0x4000000U)
#define DO_SRON_TRANS    (0x8000000U)
#define DO_PATCH_L1C     (0x10000000U)
  /* KB: third possibility to DO_SRON_RAD/DO_CORR_RAD */
#define DO_KEYDATA_RAD   (0x20000000U)
  /* KB: Apply m-factors in applicator */
#define DO_MFACTOR_RAD   (0x40000000U)
#define DO_MFAC_H5_RAD   (0x80000000U)


#define SCIA_ATBD_CALIB  (DO_CORR_VIS_MEM|DO_CORR_IR_NLIN|\
                          DO_CORR_AO|DO_CORR_DARK|DO_CORR_VDARK|\
                          DO_CORR_PPG|DO_CORR_ETALON|DO_CORR_STRAY|\
                          DO_CALIB_WAVE|DO_CORR_POL|DO_CORR_RAD|DO_MASK_BDPM)

#define SCIA_SRON_CALIB  (DO_CORR_VIS_MEM|DO_CORR_IR_NLIN|DO_SRON_MEM_NLIN|\
                          DO_CORR_AO|DO_CORR_DARK|DO_CORR_VDARK|DO_SRON_DARK|\
                          DO_CORR_STRAY|DO_CORR_PPG|DO_SRON_PPG|\
                          DO_CORR_ETALON|\
                          DO_CALIB_WAVE|DO_SRON_WAVE|\
                          DO_CORR_POL|DO_CORR_RAD|\
                          DO_MASK_BDPM|DO_SRON_BDPM)

#define SCIA_PATCH_ID    "B"
#define SCIA_PATCH_NONE  ((unsigned short) 0x0U)
#define SCIA_PATCH_MEM   ((unsigned short) 0x1U)
#define SCIA_PATCH_NLIN  ((unsigned short) 0x2U)
#define SCIA_PATCH_DARK  ((unsigned short) 0x4U)
#define SCIA_PATCH_PPG   ((unsigned short) 0x8U)
#define SCIA_PATCH_STRAY ((unsigned short) 0x10U)
#define SCIA_PATCH_BASE  ((unsigned short) 0x20U)
#define SCIA_PATCH_SUN   ((unsigned short) 0x40U)
#define SCIA_PATCH_POL   ((unsigned short) 0x80U)
#define SCIA_PATCH_RAD   ((unsigned short) 0x100U)
#define SCIA_PATCH_BDPM  ((unsigned short) 0x200U)
#define SCIA_PATCH_ALL   (SCIA_PATCH_MEM|SCIA_PATCH_NLIN|SCIA_PATCH_DARK|\
                          SCIA_PATCH_STRAY|SCIA_PATCH_BASE|SCIA_PATCH_POL|\
                          SCIA_PATCH_BDPM)

/* maximum number of SCIAMACHY obvervational states */
#define MAX_NUM_CLUS      64
#define MAX_NUM_STATE     70

#define NUM_COORDS                 5     /* 4 corner and center coordinates */

/* return Longitude within range <-180, 180] */
#define LON_IN_RANGE(lon) ((lon > 180.f) ? (lon - 360.f) : (lon))
#define ILON_IN_RANGE(lon) ((lon > 18000) ? (lon - 36000) : (lon))

/* 
 * macros to perform rounding-off to various integer types
 */
#define __ROUNDf_uc(val) ({				\
     typeof(val) _val = roundf(val);		        \
     (_val < 0) ? (unsigned char) 0 :			\
	  (_val < UCHAR_MAX) ? (unsigned char) (_val) : UCHAR_MAX;	\
	  })

#define __ROUNDf_us(val) ({				\
     typeof(val) _val = roundf(val);		        \
     (_val < 0) ? (unsigned short) 0 :			\
	  (_val < USHRT_MAX) ? (unsigned short) (_val) : USHRT_MAX;	\
	  })

#define __ROUND_us(val) ({				\
     typeof(val) _val = round(val);		        \
     (_val < 0) ? (unsigned short) 0 :			\
	  (_val < USHRT_MAX) ? (unsigned short) (_val) : USHRT_MAX;	\
	  })

/*+++++ enumeration constants +++++*/
enum ctypes_def { 
     UINT8_T = 1, 
     UINT16_T, 
     INT16_T, 
     UINT32_T, 
     INT32_T, 
     FLT32_T, 
     FLT64_T 
};

/* definition of instruments and data levels */
enum nadc_instr { 
     ALL_INSTR = 0,
     GOME_LEVEL_1,
     GOME_LEVEL_2,
     SCIA_LEVEL_0,
     SCIA_LEVEL_1,
     SCIA_LEVEL_2,
     SCIA_PATCH_1,
     MERIS_LEVEL_1,
     MERIS_LEVEL_2
};

/* constants used by nadc_flip.c */
enum nadc_flip {
     NADC_FLIP_NO = 0,
     NADC_FLIP_X,
     NADC_FLIP_Y,
     NADC_FLIP_XY
};

/*+++++ Structures & Unions +++++*/
struct param_record
{
     unsigned char flag_infile;                 /* name of input file given */
     unsigned char flag_outfile;               /* name of output file given */

     unsigned char flag_check;
     unsigned char flag_show;
     unsigned char flag_version;
     unsigned char flag_silent;
     unsigned char flag_verbose;

     unsigned char flag_cloud;
     unsigned char flag_geoloc;
     unsigned char flag_geomnmx;
     unsigned char flag_period;
     unsigned char flag_pselect;
     unsigned char flag_subset;
     unsigned char flag_sunz;
     unsigned char flag_wave;

     unsigned char use_infodb;

     unsigned char write_pds;
     unsigned char write_ascii;
     unsigned char write_hdf5;
     unsigned char flag_deflate;
     unsigned char write_meta;
     unsigned char write_sql;
     unsigned char flag_sql_remove;
     unsigned char flag_sql_replace;

     unsigned char write_lv1c;

     unsigned char write_subset;
     unsigned char write_blind;
     unsigned char write_stray;
     unsigned char write_aux0;
     unsigned char write_pmd0;
     unsigned char write_aux;
     unsigned char write_det;
     unsigned char write_pmd;
     unsigned char write_pmd_geo;
     unsigned char write_polV;

     unsigned char write_ads;
     unsigned char write_gads;

     unsigned char write_limb;
     unsigned char write_moni;
     unsigned char write_moon;
     unsigned char write_nadir;
     unsigned char write_occ;
     unsigned char write_sun;

     unsigned char write_bias;
     unsigned char write_cld;
     unsigned char write_doas;

     unsigned char catID_nr;   
     unsigned char stateID_nr;   
     unsigned char clusID_nr;

     unsigned char chan_mask;

     unsigned short patch_scia;
     unsigned short calib_earth;
     unsigned short calib_limb;
     unsigned short calib_moon;
     unsigned short calib_sun;
     unsigned short calib_pmd;
     unsigned int   calib_scia;

     int   hdf_file_id;
     int   hdf_sd_id;

     unsigned long long clus_mask;

     unsigned char  catID[MAX_NUM_STATE];
     unsigned char  stateID[MAX_NUM_STATE];

     char  bgn_date[GOME_DATE_LENGTH];
     char  end_date[GOME_DATE_LENGTH];

     char  pselect[MAX_STRING_LENGTH];

     char  program[MAX_STRING_LENGTH];
     char  infile[MAX_STRING_LENGTH];
     char  outfile[MAX_STRING_LENGTH];
     char  hdf5_name[MAX_STRING_LENGTH];

     float cloud[2];
     float geo_lat[2];
     float geo_lon[2];
     float sunz[2];
     float wave[2];
};

#define MAX_ADAGUC_INFILES   256
struct param_adaguc
{
     unsigned char flag_show;
     unsigned char flag_version;
     unsigned char flag_silent;
     unsigned char flag_verbose;

     unsigned char flag_indir;
     unsigned char flag_outdir;
     unsigned char flag_clip;

     char prodClass[5];

     char clipStart[16];
     char clipStop[16];

     unsigned short num_infiles;
     char *name_infiles[MAX_ADAGUC_INFILES];

     char indir[MAX_STRING_LENGTH];
     char outdir[MAX_STRING_LENGTH];
};

/* common Envisat PDS data structures */
struct mjd_envi
{
     int days;
     unsigned int secnd;
     unsigned int musec;
};

struct coord_envi
{
     int lat;
     int lon;
};

struct mph_envi
{
     char  product[ENVI_FILENAME_SIZE];
     char  proc_stage[2];
     char  ref_doc[24];

     char  acquis[21];
     char  proc_center[7];
     char  proc_time[MAX_UTC_STRING];
     char  soft_version[15];

     char  sensing_start[MAX_UTC_STRING];
     char  sensing_stop[MAX_UTC_STRING];

     char  phase[2];
     short cycle;
     int   rel_orbit;
     int   abs_orbit;
     char  state_vector[MAX_UTC_STRING];
     double delta_ut;
     double x_position;
     double y_position;
     double z_position;
     double x_velocity;
     double y_velocity;
     double z_velocity;
     char  vector_source[3];

     char  utc_sbt_time[MAX_UTC_STRING];
     unsigned int sat_binary_time;
     unsigned int clock_step;

     char  leap_utc[MAX_UTC_STRING];
     short leap_sign;
     char  leap_err[2];

     char  product_err[2];
     unsigned int tot_size;
     unsigned int sph_size;
     unsigned int num_dsd;
     unsigned int dsd_size;
     unsigned int num_data_sets;
};

struct dsd_envi
{
     char name[29];
     char type[2];
     char flname[ENVI_FILENAME_SIZE];
     unsigned int offset;
     unsigned int size;
     unsigned int num_dsr;
     int dsr_size;
};

struct state_list_rec
{
     unsigned char stateID;
     unsigned char intg_time;
     unsigned short numObs;
};

/* Prototypes and macros definitions for error handling */
#include <nadc_error.h>

/* Prototypes for the Netherlands DataCenter modules */
#include <proto_nadc.h>

#if (defined _GOME_COMMON)
#include <defs_gome.h>
#include <proto_gome.h>
#endif

#if (defined _SCIA_LEVEL_0)
#define _SCIA_COMMON
#endif
#if (defined _SCIA_LEVEL_1) || (defined _SCIA_PATCH_1)
#define _SCIA_COMMON
#endif
#if (defined _SCIA_LEVEL_2)
#define _SCIA_COMMON
#endif
#if (defined _SCIA_COMMON)
#include <defs_scia.h>
#include <proto_scia.h>
#endif

#if (defined _MERIS_COMMON)
#include <defs_meris.h>
#include <proto_meris.h>
#endif

#ifdef _WITH_SQL
#include <proto_sql_nadc.h>
#endif

#ifdef __cplusplus
  }
#endif
#endif   /* __DEFS_NADC */
