/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 1999 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   PROTO_GOME
.AUTHOR      R.M. van Hees 
.KEYWORDS    header file
.LANGUAGE    ANSI-C
.PURPOSE     prototypes of GOME modules
.ENVIRONment none
.VERSION     2.0     21-Aug-2001   combined level 1b and 2 modules, RvH
             1.2     16-May-2001   Added GOME_LV1_WR_ASCII_REC, RvH
             1.1     18-Feb-2000   Added/Renamed several modules, RvH
             1.0     03-Feb-1999   Creation by R.M. van Hees
------------------------------------------------------------*/

#ifndef  NADC_PROTO_GOME                           /* Avoid redefinitions */
#define  NADC_PROTO_GOME

extern void GOMEget_Version( /*@out@*/ unsigned int *, 
			     /*@out@*/ unsigned int *, 
			     /*@out@*/ unsigned int *, 
			     /*@out@*/ char * );

extern void GOME_LV1_CHK_SIZE( const struct fsr1_gome, const char * );
extern void GOME_LV2_CHK_SIZE( const struct fsr2_gome, const char * );

#if defined _STDIO_H || defined __STDIO_H__
extern void GOMEshow_Version( FILE *stream, const char * )
     /*@modifies stream@*/;

extern void GOME_RD_PIR( FILE *fp, /*@out@*/ struct pir_gome *pir )
     /*@globals  errno;@*/
     /*@modifies errno, fp, pir@*/;
extern void GOME_LV1_RD_FSR( FILE *fp, /*@out@*/ struct fsr1_gome *fsr )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *fsr@*/;
extern void GOME_LV1_RD_SPH( FILE *fp, const struct fsr1_gome *,
			     /*@out@*/ struct sph1_gome *sph )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *sph@*/;
extern short GOME_LV1_RD_FCD( FILE *fp, const struct fsr1_gome *, 
			      /*@out@*/ struct fcd_gome *fcd )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *fcd@*/;
extern short GOME_LV1_RD_PCD( FILE *fp, const struct fsr1_gome *, 
			      const struct sph1_gome *,
			      /*@out@*/ struct pcd_gome **pcd )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *pcd@*/;
extern short GOME_LV1_RD_SMCD( unsigned char, FILE *fp, 
			       const struct fsr1_gome *,
			       const struct sph1_gome *, 
			       /*@out@*/ struct smcd_gome **smcd )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *smcd@*/;
extern short GOME_LV1_RD_BDR( FILE *fp, short, const struct fsr1_gome *,
			      const struct fcd_gome *, 
			      /*@out@*/ struct rec_gome **rec )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *rec@*/;
extern void GOME_LV2_RD_FSR( FILE *fp, /*@out@*/ struct fsr2_gome *fsr )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *fsr@*/;
extern void GOME_LV2_RD_SPH( FILE *fp, const struct fsr2_gome *,
                             /*@out@*/ struct sph2_gome *sph )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *sph@*/;
extern short GOME_LV2_RD_DDR( FILE *fp, const struct fsr2_gome *,
			      const struct sph2_gome *, 
			      /*@out@*/ struct ddr_gome **ddr )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fp, *ddr@*/;
#endif   /* ---- defined _STDIO_H || defined __STDIO_H__ ----- */

extern void CALIB_PCD_PMD( unsigned char, unsigned short, 
			   const struct fcd_gome *,
			   short, const short *, struct pcd_gome * );
extern void CALIB_SMCD_PMD( unsigned short, const struct fcd_gome *,
			    short, const short *, struct smcd_gome *  );
extern void CALIB_PCD_BDR( unsigned short, short,
			   const struct fcd_gome *, 
			   short, const struct pcd_gome *, 
			   short, struct rec_gome *rec )
     /*@globals  nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack, rec@*/;
extern void CALIB_SMCD_BDR( unsigned short, short,
			    const struct fcd_gome *, 
			    const struct smcd_gome *, 
			    short, struct rec_gome *rec )
     /*@globals  nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack, rec@*/;

extern short BandChannel( short, short );
extern int   SELECT_BAND( short, struct param_record, 
			  const struct fcd_gome * )
     /*@globals  nadc_err_stack;@*/
     /*@modifies nadc_err_stack@*/;
extern short SELECT_PCD( struct param_record, const struct pcd_gome * )
     /*@globals  stderr, nadc_err_stack;@*/
     /*@modifies stderr, nadc_err_stack@*/;
extern short SELECT_SMCD( struct param_record, const struct smcd_gome * );
extern short SELECT_DDR( struct param_record, const struct glr2_gome * )
     /*@globals  nadc_err_stack;@*/
     /*@modifies nadc_err_stack@*/;

extern void GOME_LV1_PMD_GEO( unsigned char, short, const short *, 
			     struct pcd_gome *pcd )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, pcd@*/;

extern void PROCESS_PCD_BDR( short, struct param_record, 
			     const struct fsr1_gome *, 
			     const struct fcd_gome *, 
			     short, /*@null@*/ const short *, 
                             /*@null@*/ const struct pcd_gome *, 
		             const struct rec_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void PROCESS_SMCD_BDR( unsigned char, short, struct param_record, 
			      const struct fcd_gome *, short, const short *,
			      /*@null@*/ const struct smcd_gome *, 
			      const struct rec_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;

extern void GOME_LV1_WR_ASCII_FSR( struct param_record, 
				   const struct fsr1_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_ASCII_SPH( struct param_record, 
				   const struct sph1_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_ASCII_FCD( struct param_record, 
				   const struct fcd_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_ASCII_PCD( struct param_record, short, const short *,
				   const struct pcd_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_ASCII_SMCD( unsigned char, struct param_record, short, 
                              const short *, const struct smcd_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_ASCII_REC( unsigned char, short, struct param_record, 
			     short, short, short, const struct rec_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;

extern void GOME_LV2_WR_ASCII_FSR( struct param_record, 
				   const struct fsr2_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV2_WR_ASCII_SPH( struct param_record, 
				   const struct sph2_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void GOME_LV2_WR_ASCII_DDR( struct param_record, struct sph2_gome,
				   short, const struct ddr_gome * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack@*/;

#ifdef _HDF5_H
extern void GOME_WR_H5_VERSION( hid_t )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void CRE_GOME_LV1_H5_STRUCTS( struct param_record );
extern void GOME_WR_H5_PIR( struct param_record, const struct pir_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_H5_SPH( struct param_record, 
				const struct sph1_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_H5_FCD( struct param_record, 
				const struct fcd_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_H5_PCD( struct param_record, 
				short, /*@null@*/ const short *,
				/*@null@*/ const struct pcd_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_H5_SMCD( unsigned char, struct param_record, short,
				 /*@null@*/ const short *, 
				 /*@null@*/ const struct smcd_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV1_WR_H5_REC( unsigned char, short, struct param_record,
				short, short, short, const struct rec_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;

extern void GOME_LV2_WR_H5_SPH( struct param_record, 
				const struct sph2_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV2_WR_H5_DDR( struct param_record, 
				short, const struct ddr_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void GOME_LV2_WR_H5_IRR( struct param_record,
				short, const struct ddr_gome * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
#endif /* _HDF5_H */
#endif /* NADC_PROTO_GOME */
