/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 1999 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   proto_NADC.h
.AUTHOR      R.M. van Hees 
.KEYWORDS    prototypes
.LANGUAGE    ANSI-C
.PURPOSE     prototypes for the Netherlands SCIAMACHY DataCenter modules

.ENVIRONment none

.VERSION     1.0     20-May-1999   Creation by R.M. van Hees
------------------------------------------------------------*/

#ifndef  PROTO_NADC                              /* Avoid redefinitions */
#define  PROTO_NADC

extern void NADCget_Version( /*@out@*/ unsigned int *, 
                             /*@out@*/ unsigned int *, 
			     /*@out@*/ unsigned int *, 
                             /*@out@*/ char * );

extern /*@null@*/ /*@out@*/ /*@only@*/ 
       unsigned char **ALLOC_UC2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       char **ALLOC_C2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       unsigned short **ALLOC_US2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       short **ALLOC_S2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       short ***ALLOC_S3D( size_t, size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       int **ALLOC_I2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       float **ALLOC_R2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       double **ALLOC_D2D( size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       int ***ALLOC_I3D( size_t, size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@out@*/ /*@only@*/ 
       int ****ALLOC_I4D( size_t, size_t, size_t, size_t )
     /*@globals nadc_stat, nadc_err_stack;@*/
     /*@modifies nadc_stat, nadc_err_stack@*/;
extern void FREE_2D( /*@notnull@*/ /*@only@*/ void **p ) 
     /*@modifies p, *p@*/;
extern void FREE_3D( /*@notnull@*/ /*@only@*/ void ***p ) 
     /*@modifies p, *p, **p@*/;
extern void FREE_4D( /*@notnull@*/ /*@only@*/ void ****p ) 
     /*@modifies p, *p, **p, ***p@*/;

extern int BinarySearch( int, const int *, const int *, int );

extern void NADC_INTERPOL( float, float, float, unsigned int, const float *, 
			   const float *, /*@unique@*/ float *Y )
      /*@globals  nadc_stat, nadc_err_stack;@*/
      /*@modifies nadc_stat, nadc_err_stack, Y@*/;
extern void NADC_INTERPOL_d( float, float, float, unsigned int, const float *, 
			   const float *, /*@unique@*/ double *Y )
      /*@globals  nadc_stat, nadc_err_stack;@*/
      /*@modifies nadc_stat, nadc_err_stack, Y@*/;

extern short  SELECTs( const size_t, const size_t, const short  * );
extern int    SELECTi( const size_t, const size_t, const int    * );
extern float  SELECTr( const size_t, const size_t, const float  * );
extern double SELECTd( const size_t, const size_t, const double * );

#ifndef export_IDL_DEF
extern size_t strlcpy( /*@out@*/ char *, /*@unique@*/ const char *, size_t );
extern size_t strlcat( /*@out@*/ char *, /*@unique@*/ const char *, size_t );
#endif
extern void rstrip( /*@out@*/ char *, /*@unique@*/ const char * );

extern void Set_Bit_LL( unsigned long long *, unsigned char );
extern unsigned long long Get_Bit_LL( unsigned long long, unsigned char );

extern void Julian_2_MJD( double, /*@out@*/ double *, 
			  /*@out@*/ unsigned int *, /*@out@*/ unsigned int * );
extern void MJD_2_Julian( double, unsigned int, unsigned int, 
			  /*@out@*/ double * );
extern void ASCII_2_UTC( const char *, 
			/*@out@*/ unsigned int *, /*@out@*/ unsigned int * )
     /*@globals errno;@*/;
extern void ASCII_2_MJD( const char ASCII_DateTime[], /*@out@*/ int *, 
			 /*@out@*/ unsigned int *, /*@out@*/ unsigned int * )
     /*@globals errno;@*/;
extern double DATETIME_2_JULIAN( const char *, unsigned int )
     /*@globals errno;@*/;
extern void UTC_2_ASCII( unsigned int, unsigned int, /*@out@*/ char * );
extern void UTC_2_DATETIME( unsigned int, unsigned int, /*@out@*/ char * );
extern void MJD_2_ASCII( int, unsigned int, unsigned int, /*@out@*/ char * );
extern void MJD_2_DATETIME( int, unsigned int, unsigned int, /*@out@*/char * );
extern void MJD_2_YMD( int, unsigned int, /*@out@*/ char * );
extern void GomeJDAY2adaguc( double, /*@out@*/ char *dateTime );
extern double Adaguc2gomeJDAY( const char * );
extern void SciaJDAY2adaguc( double, /*@out@*/ char *dateTime );
extern double Adaguc2sciaJDAY( const char * );

extern void NADC_AKIMA_SU( int, int, size_t, const void *, const void *, 
		    /*@out@*/ double *a_coef, /*@out@*/ double *b_coef, 
		    /*@out@*/ double *c_coef, /*@out@*/ double *d_coef )
      /*@globals  nadc_stat, nadc_err_stack;@*/
      /*@modifies nadc_stat, nadc_err_stack, a_coef, b_coef, c_coef, d_coef@*/;
extern double NADC_AKIMA_PO( size_t, const double *, const double *, 
			     const double *, const double *, const double *, 
			     double );
extern void  FIT_GRID_AKIMA( int, int, size_t, const void *, const void *, 
			     int, int, size_t, const void *, 
			     /*@out@*/ void *yres )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, yres@*/;

extern void NADC_FIT( size_t, const float *, const float *, const float *,
		      /*@out@*/ float *fit_a, 
		      /*@null@*/ /*@out@*/ float *fit_b, 
		      /*@out@*/ float *sig_a, /*@out@*/ float *sig_b, 
		      /*@out@*/ float *chisq, 
		      /*@null@*/ /*@out@*/ float *q_fit  )
       /*@globals  errno;@*/
       /*@modifies errno, *fit_a, *fit_b, *sig_a, *sig_b, *chisq, *q_fit@*/;

extern void NADC_RECEIVEDATE( const char *, /*@out@*/ char *datetime )
       /*@modifies datetime@*/;

extern bool NADC_CHECK_FOR_SAA( const double, const double );

extern unsigned int NADC_FILESIZE( const char * );

extern int NADC_FILES_EQUAL( const char *, const char * )
       /*@globals  errno;@*/
       /*@modifies errno@*/;

extern void NADC_FLIPc( enum nadc_flip, const unsigned int *, signed char *matrix )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies matrix, nadc_stat, nadc_err_stack@*/;
extern void NADC_FLIPu( enum nadc_flip, const unsigned int *, unsigned char *matrix )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies matrix, nadc_stat, nadc_err_stack@*/;
extern void NADC_FLIPs( enum nadc_flip, const unsigned int *, short *matrix )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies matrix, nadc_stat, nadc_err_stack@*/;
extern void NADC_FLIPr( enum nadc_flip, const unsigned int *, float *matrix )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies matrix, nadc_stat, nadc_err_stack@*/;

extern unsigned short NADC_GOME_CalibMask( /*@notnull@*/ const char * );
extern unsigned short NADC_GOME_CalibStr( unsigned short, /*@out@*/ char * );

extern unsigned short NADC_SCIA_PatchMask( /*@notnull@*/ const char * );
extern unsigned short NADC_SCIA_PatchStr( unsigned short, /*@out@*/ char * );

extern unsigned int   NADC_SCIA_CalibMask( /*@notnull@*/ const char * );
extern unsigned short NADC_SCIA_CalibStr( unsigned int, /*@out@*/ char * );

extern void NADC_SHOW_PARAM( int, struct param_record );

extern void ADAGUC_INIT_PARAM( int, char **, 
			       /*@out@*/ struct param_adaguc *param )
     /*@globals  errno, stderr, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, stderr, nadc_stat, nadc_err_stack, param@*/;

extern void NADC_INIT_PARAM( /*@out@*/ struct param_record *param )
     /*@globals  errno, stderr, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, stderr, nadc_stat, nadc_err_stack, param@*/;

extern void NADC_SET_PARAM( int, char **, int,
                             /*@out@*/ struct param_record *param )
     /*@globals  errno, stderr, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, stderr, nadc_stat, nadc_err_stack, param@*/;

extern void NADC_USRINP( int , /*@unique@*/ const char *, int , 
			/*@out@*/ void *pntr, /*@out@*/ int *nrval )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, *nrval, *pntr@*/;

extern short NADC_USRINDX( const char *, int, /*@out@*/ short *indices )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, *indices@*/;

extern void ENVI_WR_ASCII_MPH( struct param_record, 
			       const struct mph_envi * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack@*/;
extern void ENVI_WR_ASCII_DSD( struct param_record, unsigned int, 
			       const struct dsd_envi * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack@*/;

extern unsigned int GET_ENVI_DSD_INDX( unsigned int, const struct dsd_envi *,
				       const char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;

#if defined _STDIO_INCLUDED || defined _STDIO_H || defined __STDIO_H__
extern void ADAGUCshow_Version( FILE *, const char * );

extern unsigned int ENVI_RD_PDS_INFO( FILE *fp, /*@out@*/ char *keyword, 
                                      /*@out@*/ char *keyvalue )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fp, keyword, keyvalue@*/;

extern void ENVI_RD_MPH( FILE *fp, /*@out@*/ struct mph_envi *mph )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fp, *mph@*/;
extern void ENVI_WR_MPH( FILE *fp, const struct mph_envi )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fp@*/;
extern unsigned int ENVI_RD_DSD( FILE *fp, const struct mph_envi,
				 /*@out@*/ struct dsd_envi *dsd_out )
       /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fp, *dsd_out@*/;
extern void ENVI_WR_DSD( FILE *fp, const unsigned int,
			 const struct dsd_envi *dsd_out )
       /*@globals  errno;@*/
       /*@modifies errno, fp@*/;

extern void NADC_CopyRight( FILE *stream )
     /*@modifies stream@*/;
extern void NADC_SCIA_PatchShow( FILE *stream )
     /*@modifies stream@*/;
extern void NADC_GOME_CalibShow( FILE *stream )
     /*@modifies stream@*/;
extern void NADC_SCIA_CalibShow( FILE *stream )
     /*@modifies stream@*/;
extern void NADC_Info_Proc( FILE *stream, const char *, unsigned int )
     /*@modifies stream@*/;
extern void NADC_Info_Update( FILE *stream, unsigned short, unsigned int )
     /*@modifies stream@*/;
extern void NADC_Info_Finish( FILE *stream, unsigned short, unsigned int )
     /*@modifies stream@*/;
extern /*@null@*/ /*@dependent@*/
       FILE *CRE_ASCII_File( const char *, const char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern /*@null@*/ /*@dependent@*/
       FILE *CAT_ASCII_File( const char *, const char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void nadc_write_text( FILE *fp, unsigned int, const char *, 
			     const char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_bool( FILE *fp, unsigned int, const char *, bool )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_schar( FILE *fp, unsigned int, const char *, 
			      signed char )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_uchar( FILE *fp, unsigned int, const char *, 
			      unsigned char )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_short( FILE *fp, unsigned int, const char *,  short )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_ushort( FILE *fp, unsigned int, const char *, 
			       unsigned short )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_int( FILE *fp, unsigned int, const char *, int )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_uint( FILE *fp, unsigned int, const char *, 
			     unsigned int )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_long( FILE *fp, unsigned int, const char *, long )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_float( FILE *fp, unsigned int, const char *, int, 
			      float )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_double( FILE *fp, unsigned int, const char *, int, 
			       double )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_schar( FILE *fp, unsigned int, const char *, int, 
				  const unsigned int *, const signed char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_uchar( FILE *fp, unsigned int, const char *, int, 
				  const unsigned int *, 
				  const unsigned char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_short( FILE *fp, unsigned int, const char *, int, 
				  const unsigned int *, const short * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_ushort( FILE *fp, unsigned int, const char *, int, 
				   const unsigned int *, 
				   const unsigned short * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_int( FILE *fp, unsigned int, const char *, int, 
				const unsigned int *, const int * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_uint( FILE *fp, unsigned int, const char *, int, 
				 const unsigned int *, const unsigned int * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_float( FILE *fp, unsigned int, const char *, int, 
				  const unsigned int *, int, const float * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_arr_double( FILE *fp, unsigned int, const char *, int, 
				   const unsigned int *, int, const double *)
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;
extern void nadc_write_header( FILE *fp, unsigned int, const char *, 
			       const char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies fp, nadc_stat, nadc_err_stack@*/;

extern void NADC_GET_XML_METADB( FILE *stream, /*@out@*/ char *, 
				 /*@out@*/ char *, /*@out@*/ char *,
				 /*@out@*/ char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies stream, nadc_stat, nadc_err_stack@*/;

#endif   /* ---- defined _STDIO_INCLUDED || defined _STDIO_H ----- */

#ifdef _HDF5_H
extern void Create_HDF5_NADC_FILE( int, struct param_record *param )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, param->hdf_file_id, param->hdf_sd_id*/;

extern hid_t NADC_OPEN_HDF5_Group( hid_t, const char * );

extern void NADC_WR_HDF5_Attribute( hid_t, const char *, hid_t,
				    int, const hsize_t *, const void * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;

extern void NADC_RD_HDF5_Dataset( hid_t, const char *, hid_t, 
				  /*@out@*/ int *rank, 
				  /*@out@*/ hsize_t *dims, 
                                  /*@out@*/ void **data_out )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, rank, dims, *data_out*/;

extern void NADC_WR_HDF5_Dataset( hbool_t, hid_t, const char *,
				  hid_t, int, const hsize_t *, const void * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;

extern void NADC_WR_HDF5_Vlen_Dataset( hbool_t, hid_t, const char *, 
				       hid_t, int, const hsize_t *, 
				       /*@only@*/ void * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;

extern void NADC_CRE_H5_EArray_uint8( hid_t, const char *,
				      const hsize_t, int, 
				      const unsigned char * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_CRE_H5_EArray_uint16( hid_t, const char *,
				       const hsize_t, int,
				       const unsigned short * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_CRE_H5_EArray_int32( hid_t, const char *,
				      const hsize_t, int,
				      const int * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_CRE_H5_EArray_uint32( hid_t, const char *,
				       const hsize_t, int,
				       const unsigned int * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_CRE_H5_EArray_float( hid_t, const char *,
				      const hsize_t, int, const float * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_CRE_H5_EArray_struct( hid_t, const char *, const hsize_t, 
				       int, hid_t, const void * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_CAT_H5_EArray( hid_t, const char *, 
				int, size_t, const void * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;
extern void NADC_WR_H5_EArray( hid_t, const char *, 
			       size_t, size_t, const void * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack*/;

extern void NADC_RD_H5_EArray_uint8( hid_t, const char *, hsize_t, int, 
				     const int *, 
				     /*@out@*/ unsigned char *buffer )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, buffer*/;
extern void NADC_RD_H5_EArray_uint16( hid_t, const char *, hsize_t, int, 
				      const int *, 
				      /*@out@*/ unsigned short *buffer )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, buffer*/;
extern void NADC_RD_H5_EArray_int32( hid_t, const char *, hsize_t, int, 
				     const int *, /*@out@*/ int *buffer )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, buffer*/;
extern void NADC_RD_H5_EArray_uint32( hid_t, const char *, hsize_t, int, 
				      const int *, 
				      /*@out@*/ unsigned int *buffer )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, buffer*/;
extern void NADC_RD_H5_EArray_float( hid_t, const char *, hsize_t, int, 
				     const int *, /*@out@*/ float *buffer )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, buffer*/;

extern hid_t PYTABLE_open_file( const char *, const char * );
extern hid_t PYTABLE_open_group( hid_t, const char * );
extern herr_t PYTABLE_make_array( hid_t, const char *, const char *,
				  const int, const hsize_t *, int, hid_t, 
				  const hsize_t *, /*@null@*/ void *,
				  unsigned int, bool, bool, const void * );
extern herr_t PYTABLE_append_array( hid_t, const char *, 
				    int, int, const void * );
extern herr_t PYTABLE_write_records( hid_t, const char *, hsize_t *, hsize_t *,
				     hsize_t *, const void * );
#endif   /* ---- defined _HDF5_H ----- */

#endif  /* PROTO_NADC */
