/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 1999 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.

.IDENTifer   PROTO_SCIA
.AUTHOR      R.M. van Hees 
.KEYWORDS    header file
.LANGUAGE    ANSI-C
.PURPOSE     prototypes of SCIAMACHY modules
.ENVIRONment none
.VERSION     2.0     23-May-2001   Starting adding HDF4 routines, RvH
             1.0     12-Aug-1999   Creation by R.M. van Hees
------------------------------------------------------------*/

#ifndef  NADC_PROTO_SCIA                           /* Avoid redefinitions */
#define  NADC_PROTO_SCIA

extern void SCIAget_Version( /*@out@*/ unsigned int *, 
                             /*@out@*/ unsigned int *, 
			     /*@out@*/ unsigned int *, 
                             /*@out@*/ char * );

extern unsigned int GET_SCIA_MAGIC_ID( const char * )
       /*@globals errno;@*/
       /*@modifies errno@*/;

extern unsigned short GET_SCIA_QUALITY( int, /*@null@*/ /*@out@*/ int *period );

extern void GET_SCIA_ROE_INFO( bool, const double, /*@out@*/ int *, 
			       /*@out@*/ bool *, /*@out@*/ float * );
extern double GET_SCIA_ROE_JDAY( unsigned short );

extern void GET_SCIA_MEAN_STATE( bool, 
				 unsigned short, const struct mds1c_scia *,
				 /*@out@*/ unsigned short *, 
				 /*@out@*/ unsigned char *,  /*@out@*/ float *,
				 /*@out@*/ float *, /*@out@*/ float * );

extern void SCIA_LV1C_FREE_MDS( int, unsigned int, 
				/*@only@*/ struct mds1c_scia * );
extern void SCIA_LV1C_FREE_MDS_PMD( int, /*@only@*/ struct mds1c_pmd * );
extern void SCIA_LV1C_FREE_MDS_POLV( int, /*@only@*/ struct mds1c_polV * );

extern void SCIA_LV1C_CAL( int, unsigned int, unsigned short, 
			   struct mds1c_scia *mds_out )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, mds_out->pixel_val@*/;

#if defined _STDIO_INCLUDED || defined _STDIO_H || defined __STDIO_H__
extern void SCIAshow_Version( FILE *stream, const char * )
     /*@modifies stream@*/;

extern unsigned int SCIA_RD_LADS( FILE *fp, unsigned int, 
				  const struct dsd_envi *,
				  /*@out@*/ struct lads_scia **lads )
       /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack, fp, *lads@*/;
#endif   /* ---- defined _STDIO_INCLUDED || defined _STDIO_H ----- */

extern void SCIA_WR_ASCII_LADS( struct param_record, unsigned int, 
				const struct lads_scia * )
       /*@globals  errno, nadc_stat, nadc_err_stack;@*/
       /*@modifies errno, nadc_stat, nadc_err_stack@*/;

#ifdef _HDF5_H
extern void SCIA_WR_H5_VERSION( hid_t )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void SCIA_WR_H5_MPH( struct param_record, const struct mph_envi * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern unsigned int SCIA_RD_H5_LADS( struct param_record, struct lads_scia * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void SCIA_WR_H5_LADS( struct param_record, unsigned int,
			     const struct lads_scia * )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
#endif /* _HDF5_H */

#if (defined _SCIA_LEVEL_0)
#include <proto_scia_lv0.h>
#endif

#if (defined _SCIA_LEVEL_1) || (defined _SCIA_PATCH_1)
#include <proto_scia_lv0.h>
#include <proto_scia_lv1.h>
#if (defined _SCIA_PATCH_1)
#include <proto_scia_patch.h>
#endif
#endif

#if (defined _SCIA_LEVEL_2)
#include <proto_scia_lv2.h>
#endif

#endif /* NADC_PROTO_SCIA */
