/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2011 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA  02111-1307, USA.

.IDENTifer   PROTO_SCIA_CAL
.AUTHOR      R.M. van Hees 
.KEYWORDS    header file
.LANGUAGE    ANSI-C
.PURPOSE     prototypes of the specific SCIA calibation modules
.ENVIRONment none
.VERSION     1.0     17-Oct-2011   Creation by R.M. van Hees
------------------------------------------------------------*/

#ifndef  NADC_PROTO_SCIA_CAL                       /* Avoid redefinitions */
#define  NADC_PROTO_SCIA_CAL

extern void SCIA_RD_H5_MEM( /*@out@*/ struct scia_memcorr *mem )
       /*@globals nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void SCIA_FREE_H5_MEM( struct scia_memcorr *mem );
extern void SCIA_RD_H5_NLIN( /*@null@*/ const char *file, 
			     /*@out@*/ struct scia_nlincorr *nlin )
       /*@globals nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void SCIA_FREE_H5_NLIN( struct scia_nlincorr *nlin );
extern void SCIA_RD_H5_STRAY( /*@out@*/ struct scia_straycorr *stray )
       /*@globals nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack@*/;
extern void SCIA_FREE_H5_STRAY( struct scia_straycorr *stray );

/* #if defined _STDIO_INCLUDED || defined __STDIO_H__ */
#if defined _STDIO_H || defined S_SPLINT_S
extern void SCIA_LV1_CAL( FILE *fp, unsigned int,
			  const struct state1_scia *,
			  const struct mds1_scia *, 
			  struct mds1c_scia *mds_1c )
     /*@globals errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc, 
                internalState;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc, 
                 internalState, fp, mds_1c->rad_units_flag, 
		 mds_1c->pixel_wv, mds_1c->pixel_wv_err, 
		 mds_1c->pixel_val, mds_1c->pixel_err@*/;

extern void SCIA_LV1_PATCH_MDS( FILE *fp, unsigned short, 
				const struct state1_scia *, 
				struct mds1_scia *mds )
      /*@globals  errno, stderr, nadc_stat, nadc_err_stack, internalState;@*/
      /*@modifies errno, stderr, nadc_stat, nadc_err_stack, fp, mds, 
	          internalState@*/;

extern void SCIA_LV1C_SCALE( unsigned int, struct mds1c_scia *mds_1c )
       /*@globals  nadc_stat, nadc_err_stack;@*/
       /*@modifies nadc_stat, nadc_err_stack, mds_1c->pixel_val@*/;
#endif   /* ---- defined _STDIO_INCLUDED || defined _STDIO_H ----- */

extern void SCIA_LV1C_FLAG_BDPM( unsigned short, unsigned short, 
				 struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val@*/;

#if (defined _SCIA_LEVEL_1)
extern void SCIA_SRON_CAL_MEM( unsigned short, struct mds1c_scia * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val@*/;
extern void SCIA_SRON_CAL_NLIN( unsigned short, struct mds1c_scia * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val@*/;

extern void SCIA_ATBD_FLAG_BDPM( const struct file_rec *fileParam,
				 const struct state1_scia *, 
				 struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val@*/;

extern void SCIA_SRON_FLAG_BDPM( const struct file_rec *fileParam,
				 const struct state1_scia *, 
				 struct mds1c_scia * );
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val@*/;

extern void SCIA_ATBD_CAL_DARK( const struct file_rec *fileParam,
				const struct state1_scia *,
				struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;
extern void SCIA_SRON_CAL_DARK( const struct file_rec *fileParam,
				const struct state1_scia *,
				struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;
extern void SCIA_STATE_CAL_DARK( const struct file_rec *fileParam,
				 const struct state1_scia *,
				 struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;

extern void SCIA_ATBD_CAL_ETALON( const struct file_rec *,
				  const struct state1_scia *, 
				  struct mds1c_scia * );

extern void SCIA_ATBD_CAL_MEM( unsigned char,
			       const struct state1_scia *,
			       const struct mds1_scia *,
			       struct mds1c_scia * );

extern void SCIA_ATBD_CAL_NLIN( unsigned char,
				const struct state1_scia *,
				const struct mds1_scia *,
				struct mds1c_scia * );

extern void SCIA_ATBD_CAL_STRAY( float, const struct state1_scia *, 
				 const struct mds1_scia *,
				 struct mds1c_scia * );

extern void SCIA_SRON_CAL_NOISE( const struct file_rec *,
				 const struct state1_scia *, 
				 struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, mds_1c->pixel_err@*/;

extern void SCIA_SRON_CAL_TRANS( const struct file_rec *,
				 const struct state1_scia *, 
				 struct mds1c_scia * )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val@*/;

extern void SCIA_ATBD_CAL_PPG( const struct file_rec *fileParam,
			       const struct state1_scia *, 
			       struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;
extern void SCIA_SRON_CAL_PPG( const struct file_rec *fileParam,
			       const struct state1_scia *, 
			       struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;

extern void SCIA_ATBD_CAL_POL( const struct file_rec *,
			       const struct wvlen_rec ,
			       const struct state1_scia *, 
			       const struct mds1_scia *,
			       struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;

extern void SCIA_ATBD_CAL_RAD( const struct file_rec *,
			       const struct wvlen_rec,
			       const struct state1_scia *,
			       struct mds1c_scia * )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       mds_1c->pixel_val, mds_1c->pixel_err@*/;

extern void SCIA_ATBD_CAL_REFL( const struct file_rec *,
				const struct state1_scia *,
				struct mds1c_scia * )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/;

extern void SCIA_SRON_CAL_REFL( const struct file_rec *fileParam,
                                const struct state1_scia *state,
                                struct mds1c_scia *mds_1c )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/;

extern void SCIA_ATBD_CAL_WAVE( const struct wvlen_rec, 
				const struct state1_scia *,
				struct mds1c_scia * );
#endif

extern void SCIA_SMR_CAL_RAD( unsigned short, unsigned short, 
			      float, float, /*@null@*/ const float *, 
			      /*@out@*/ float *smr )
      /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
      /*@modifies errno, nadc_stat, nadc_err_stack, smr@*/;

extern void SCIA_ATBD_INIT_WAVE( const struct file_rec *fileParam,
				 float, struct wvlen_rec *wvlen )
     /*@globals  errno, nadc_stat, nadc_err_stack, Use_Extern_Alloc;@*/
     /*@modifies errno, nadc_stat, nadc_err_stack, fileParam->fp,
       wvlen->science, wvlen->error, wvlen->solar@*/;

#endif /* NADC_PROTO_SCIA_CAL */
