/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2003 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   NADC_SHOW_PARAM
.AUTHOR      R.M. van Hees
.KEYWORDS    command-line parameter handling
.LANGUAGE    ANSI C
.PURPOSE     show command-line settings, as stored in the param-record
.INPUT/OUTPUT
  call as   NADC_SHOW_PARAM( instrument, param );
     input:
             int instrument     :   code for instrument en data product level
	     struct param_record 
                          param :   struct holding user-defined settings
.RETURNS     Nothing
.COMMENTS    None
.ENVIRONment None
.VERSION      1.1   08-Nov-2005 minor changes: ALL -> All, TRUE -> True, RvH
              1.0   28-Feb-2003 created by R. M. van Hees
------------------------------------------------------------*/
/*
 * Define _ISOC99_SOURCE to indicate
 * that this is a ISO C99 program
 */
#define  _ISOC99_SOURCE

/*+++++ System headers +++++*/
#include <stdio.h>
#include <string.h>
#include <time.h>

/*+++++ Local Headers +++++*/
#include <defs_nadc.h>
#include "../VERSION"

/*+++++ Macros +++++*/
	/* NONE */

/*+++++ Global Variables +++++*/
	/* NONE */

/*+++++++++++++++++++++++++ Main Program or Function +++++++++++++++*/
void NADC_SHOW_PARAM( int instrument, const struct param_record param )
{
     register int ii;
     register unsigned int nr = 0;

     char    cbuff[MAX_STRING_LENGTH];
     char    string[SHORT_STRING_LENGTH];
     time_t  tp[1];
     unsigned int adim, dims[2];

     FILE *outfl = stdout;
/*
 * show program name and version
 */
     (void) snprintf( string, SHORT_STRING_LENGTH, "%s (version %s)", 
		      param.program, patch_version );
     nadc_write_text( outfl, ++nr, "Program", string );
/*
 * show time of call
 */
     (void) time( tp );
     (void) strlcpy( cbuff, ctime( tp ), MAX_STRING_LENGTH );
     cbuff[strlen(cbuff)-1] = '\0';
     nadc_write_text( outfl, ++nr, "ProcessingDate", cbuff );
/*
 * show output files
 */
     nadc_write_text( outfl, ++nr, "InputFilename", param.infile );
     if ( param.write_ascii == PARAM_SET )
	  nadc_write_text( outfl, ++nr, "OutputFilename", param.outfile );
     if ( param.write_hdf5 == PARAM_SET ) {
	  nadc_write_text( outfl, ++nr, "Output filename", param.hdf5_name );
	  if ( param.flag_deflate == PARAM_SET )
	       nadc_write_text( outfl, ++nr, "Compression", "True" );
	  else
	       nadc_write_text( outfl, ++nr, "Compression", "False" );
     }
     if ( instrument == SCIA_LEVEL_1 ) {
	  if ( param.write_lv1c == PARAM_SET )
	       nadc_write_text( outfl, ++nr, "lv1cOutput", "True" );
	  else
	       nadc_write_text( outfl, ++nr, "lv1cOutput", "False" ); 
     }
/*
 * ----- General options
 */
     if ( param.flag_silent == PARAM_SET )
	  nadc_write_text( outfl, ++nr, "Silent", "True" );
     else
	  nadc_write_text( outfl, ++nr, "Silent", "False" );
/*
 * time window
 */
     if ( param.flag_period == PARAM_SET ) {
	  nadc_write_text( outfl, ++nr, "StartDate", param.bgn_date );
	  nadc_write_text( outfl, ++nr, "EndDate", param.end_date );
     }

     switch ( instrument ) {
/*
 * ----- GOME level 1b processor specific options
 */
     case GOME_LEVEL_1:
/*
 * geolocation (bounding box)
 */
	  if ( param.flag_geoloc == PARAM_SET ) {
	       dims[0] = 2;
	       nadc_write_arr_float( outfl, ++nr, "Latitude",
				      1, dims, 3, param.geo_lat );
	       nadc_write_arr_float( outfl, ++nr, "Longitude",
				      1, dims, 3, param.geo_lon );
	       if ( param.flag_geomnmx == PARAM_SET )
		    nadc_write_text( outfl, ++nr, "WithinRegion", "True" );
	       else
		    nadc_write_text( outfl, ++nr, "WithinRegion", "False" );
	  }
/*
 * selected MDS records
 */
	  (void) strcpy( cbuff, "" );
	  if ( param.write_nadir == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoNadir" );
	  }
	  if ( param.write_moon == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoMoon" );
	  }
	  if ( param.write_sun == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoSun" );
	  }
	  if ( strlen( cbuff ) == 0 ) (void) strcpy( cbuff, "All" );
	  nadc_write_text( outfl, ++nr, "MeasurementDataSets", cbuff );
/*
 * PMD data
 */
	  if ( param.write_pmd == PARAM_SET ) {
	       nadc_write_text( outfl, ++nr, "PMD", "True" );

	       if ( param.calib_pmd == CALIB_NONE ) {
		    nadc_write_text( outfl, ++nr, 
				      "PMD_Calibration", "None" );
	       } else {
		    adim = NADC_GOME_CalibStr( param.calib_pmd, string );
		    if ( adim == 0 )
			 nadc_write_text( outfl, ++nr, 
				      "PMD_Calibration", "None" );
		    else
			 nadc_write_text( outfl, ++nr, 
				      "PMD_Calibration", string );
	       }
	       if ( param.write_pmd_geo == PARAM_SET )
		    nadc_write_text( outfl, ++nr, 
				      "PMD_Geolocation", "True" );
	       else
		    nadc_write_text( outfl, ++nr, 
				      "PMD_Geolocation", "False" );
	  }
/*
 * pixel number range
 */
	  if ( param.flag_pselect == PARAM_SET ) {
	       nadc_write_text( outfl, ++nr, "GroundPixelRange", 
				 param.pselect );
	  }
/*
 * ground pixel type selection
 */
	  if ( param.flag_subset == PARAM_SET ) {
	       (void) strcpy( string, "" );
	       if ( (param.write_subset & SUBSET_EAST) != UCHAR_ZERO )
		    (void) strcat( string, "E" );
	       if ( (param.write_subset & SUBSET_CENTER) != UCHAR_ZERO )
		    (void) strcat( string, "C" );
	       if ( (param.write_subset & SUBSET_WEST) != UCHAR_ZERO )
		    (void) strcat( string, "W" );
	       if ( (param.write_subset & SUBSET_BACK) != UCHAR_ZERO )
		    (void) strcat( string, "B" );
	       nadc_write_text( outfl, ++nr, "GroundPixelType", string );
	  }
/*
 * spectral bands
 */
	  if ( (param.chan_mask & BAND_ALL) != UCHAR_ZERO ) {
	       (void) strcpy( string, "" );
	       if ( (param.chan_mask & BAND_ONE_A) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "1a" );
	       }
	       if ( (param.chan_mask & BAND_ONE_B) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "1b" );
	       }
	       if ( (param.chan_mask & BAND_TWO_A) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "2a" );
	       }
	       if ( (param.chan_mask & BAND_TWO_B) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "2b" );
	       }
	       if ( (param.chan_mask & BAND_THREE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "3" );
	       }
	       if ( (param.chan_mask & BAND_FOUR) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "4" );
	       }
	       if ( strlen( string ) == 0 )
		    nadc_write_text( outfl, ++nr, "Bands", "None" );
	       else
		    nadc_write_text( outfl, ++nr, "Bands", string );
	  }
/*
 * blind data
 */
	  if ( param.write_blind == PARAM_SET )
	       nadc_write_text( outfl, ++nr, "Blind", "True" );
	  else
	       nadc_write_text( outfl, ++nr, "Blind", "False" );
/*
 * straylight data
 */
	  if ( param.write_stray == PARAM_SET )
	       nadc_write_text( outfl, ++nr, "Straylight", "True" );
	  else
	       nadc_write_text( outfl, ++nr, "Straylight", "False" );
/*
 * wavelength range
 */
	  if ( param.flag_wave == PARAM_SET ) {
	       dims[0] = 2;
	       nadc_write_arr_float( outfl, ++nr, "wave",
				      1, dims, 3, param.wave );
	  }
/*
 * calibration of the spectral band data
 */
	  if ( param.calib_earth == CALIB_NONE ) {
	       nadc_write_text( outfl, ++nr, "BandCalibration", "None" );
	  } else {
	       char *cpntr;

	       adim = NADC_GOME_CalibStr( param.calib_earth, string );
	       if ( adim == 0 ) {
		    nadc_write_text( outfl, ++nr, 
				      "EarthCalibration", "None" );
	       } else {
		    if ( (cpntr = strchr( string, 'B' )) != NULL ) 
			 *cpntr = 'A';
		    nadc_write_text( outfl, ++nr, 
				      "EarthCalibration", string );
	       }
	  }
	  if ( param.calib_moon == CALIB_NONE ) {
	       nadc_write_text( outfl, ++nr, "MoonCalibration", "None" );
	  } else {
	       adim = NADC_GOME_CalibStr( param.calib_moon, string );
	       if ( adim == 0 )
		    nadc_write_text( outfl, ++nr, 
				      "MoonCalibration", "None" );
	       else
		    nadc_write_text( outfl, ++nr, 
				      "MoonCalibration", string );
	  }
	  if ( param.calib_sun == CALIB_NONE ) {
	       nadc_write_text( outfl, ++nr, "SunCalibration", "None" );
	  } else {
	       adim = NADC_GOME_CalibStr( param.calib_sun, string );
	       if ( adim == 0 )
		    nadc_write_text( outfl, ++nr, 
				      "SunCalibration", "None" );
	       else
		    nadc_write_text( outfl, ++nr, 
				      "SunCalibration", string );
	  }
	  break;
/*
 *  ----- GOME level 2 processor specific options
 */
     case GOME_LEVEL_2:
/*
 * geolocation (bounding box)
 */
	  if ( param.flag_geoloc == PARAM_SET ) {
	       dims[0] = 2;
	       nadc_write_arr_float( outfl, ++nr, "Latitude",
				      1, dims, 3, param.geo_lat );
	       nadc_write_arr_float( outfl, ++nr, "Longitude",
				      1, dims, 3, param.geo_lon );
	       if ( param.flag_geomnmx == PARAM_SET )
		    nadc_write_text( outfl, ++nr, "WithinRegion", "True" );
	       else
		    nadc_write_text( outfl, ++nr, "WithinRegion", "False" );
	  }
/*
 * pixel number range
 */
	  if ( param.flag_pselect == PARAM_SET ) {
	       nadc_write_text( outfl, ++nr, "GroundPixelRange", 
				 param.pselect );
	  }
/*
 * Sun zenith angle range
 */
	  if ( param.flag_sunz == PARAM_SET ) {
	       dims[0] = 2;
	       nadc_write_arr_float( outfl, ++nr, "SunZenithAngle",
				      1, dims, 3, param.sunz );
	  }
/*
 * cloud cover range
 */
	  if ( param.flag_cloud == PARAM_SET ) {
	       dims[0] = 2;
	       nadc_write_arr_float( outfl, ++nr, "CloudCover",
				      1, dims, 3, param.cloud );
	  }
	  break;
/*
 *  ----- SCIAMACHY level 0 processor specific options
 */
     case SCIA_LEVEL_0:
/*
 * selected measurement data sets
 */
	  (void) strcpy( cbuff, "" );
	  if ( param.write_aux == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoAuxiliary" );
	  }
	  if ( param.write_det == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoDetector" );
	  }
	  if ( param.write_pmd == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoPMD" );
	  }
	  if ( strlen( cbuff ) == 0 ) (void) strcpy( cbuff, "All" );
	  nadc_write_text( outfl, ++nr, "MeasurementDataSets", cbuff );
/*
 * selected States
 */
	  if ( param.stateID_nr == PARAM_UNSET ) {
	       nadc_write_text( outfl, ++nr, "StateID", "All" );
	  } else {
	       (void) sprintf( cbuff, "%-d", (int) param.stateID[0] );
	       for ( ii = 1; ii < (int) param.stateID_nr; ii++ ) {
		    (void) sprintf( cbuff, "%s,%-d", 
				    cbuff, (int) param.stateID[ii] );
	       }
	       nadc_write_text( outfl, ++nr, "StateID", cbuff );
	  }
/*
 * spectral bands
 */
	  if ( (param.chan_mask & BAND_ALL) != UCHAR_ZERO ) {
	       (void) strcpy( string, "" );
	       if ( (param.chan_mask & BAND_ONE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "1" );
	       }	  
	       if ( (param.chan_mask & BAND_TWO) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "2" );
	       }
	       if ( (param.chan_mask & BAND_THREE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "3" );
	       }
	       if ( (param.chan_mask & BAND_FOUR) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "4" );
	       }
	       if ( (param.chan_mask & BAND_FIVE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "5" );
	       }
	       if ( (param.chan_mask & BAND_SIX) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "6" );
	       }
	       if ( (param.chan_mask & BAND_SEVEN) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "7" );
	       }
	       if ( (param.chan_mask & BAND_EIGHT) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "8" );
	       }
	       if ( strlen( string ) == 0 )
		    nadc_write_text( outfl, ++nr, "Bands", "None" );
	       else
		    nadc_write_text( outfl, ++nr, "Bands", string );
	  }
	  break;
/*
 *  ----- Patch SCIAMACHY level 1 processor specific options
 */
     case SCIA_PATCH_1:
	  if ( param.patch_scia == SCIA_PATCH_NONE ) {
	       nadc_write_text( outfl, ++nr, "Patch Options", "None" );
	  } else {
	       adim = NADC_SCIA_PatchStr( param.patch_scia, string );
	       if ( adim == 0 )
		    nadc_write_text( outfl, ++nr, "Patch Options", "None" );
	       else
		    nadc_write_text( outfl, ++nr, "Patch Options", string );
	  }
	  break;
/*
 *  ----- SCIAMACHY level 1 processor specific options
 */
     case SCIA_LEVEL_1:
/*
 * geolocation (bounding box)
 */
	  if ( param.flag_geoloc == PARAM_SET ) {
	       dims[0] = 2;
	       nadc_write_arr_float( outfl, ++nr, "Latitude",
				      1, dims, 3, param.geo_lat );
	       nadc_write_arr_float( outfl, ++nr, "Longitude",
				      1, dims, 3, param.geo_lon );
	       if ( param.flag_geomnmx == PARAM_SET )
		    nadc_write_text( outfl, ++nr, "WithinRegion", "True" );
	       else
		    nadc_write_text( outfl, ++nr, "WithinRegion", "False" );
	  }
/*
 * selected Key DSD's
 */
	  (void) strcpy( cbuff, "" );
	  if ( param.write_ads == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoADS" );
	  }
	  if ( param.write_gads == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoGADS" );
	  }
	  if ( strlen( cbuff ) == 0 ) (void) strcpy( cbuff, "All" );
	  nadc_write_text( outfl, ++nr, "KeyDataSets", cbuff );
/*
 * selected measurement data sets
 */
	  (void) strcpy( cbuff, "" );
	  if ( param.write_limb == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoLimb" );
	  }
	  if ( param.write_moni == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoMonitoring" );
	  }
	  if ( param.write_nadir == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoNadir" );
	  }
	  if ( param.write_occ == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoOcc" );
	  }
	  if ( strlen( cbuff ) == 0 ) (void) strcpy( cbuff, "All" );
	  nadc_write_text( outfl, ++nr, "MeasurementDataSets", cbuff );
/*
 * PMD data
 */
	  if ( param.write_pmd == PARAM_SET )
	       nadc_write_text( outfl, ++nr, "PMD", "True" );
	  else
	       nadc_write_text( outfl, ++nr, "PMD", "None" );
/*
 * fractional polarisation data
 */
	  if ( param.write_polV == PARAM_SET )
	       nadc_write_text( outfl, ++nr, "FracPol", "True" );
	  else
	       nadc_write_text( outfl, ++nr, "FracPol", "None" );
/*
 * selected Categories
 */
	  if ( param.catID_nr == PARAM_UNSET ) {
	       nadc_write_text( outfl, ++nr, "Category", "All" );
	  } else {
	       (void) sprintf( cbuff, "%-d", (int) param.catID[0] );
	       for ( ii = 1; ii < (int) param.catID_nr; ii++ ) {
		    (void) sprintf( cbuff, "%s,%-d", 
				    cbuff, (int) param.catID[ii] );
	       }
	       nadc_write_text( outfl, ++nr, "Category", cbuff );
	  }
/*
 * selected States
 */
	  if ( param.stateID_nr == PARAM_UNSET ) {
	       nadc_write_text( outfl, ++nr, "StateID", "All" );
	  } else {
	       (void) sprintf( cbuff, "%-d", (int) param.stateID[0] );
	       for ( ii = 1; ii < (int) param.stateID_nr; ii++ ) {
		    (void) sprintf( cbuff, "%s,%-d", 
				    cbuff, (int) param.stateID[ii] );
	       }
	       nadc_write_text( outfl, ++nr, "StateID", cbuff );
	  }
/*
 * spectral bands
 */
	  if ( (param.chan_mask & BAND_ALL) != UCHAR_ZERO ) {
	       (void) strcpy( string, "" );
	       if ( (param.chan_mask & BAND_ONE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "1" );
	       }	  
	       if ( (param.chan_mask & BAND_TWO) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "2" );
	       }
	       if ( (param.chan_mask & BAND_THREE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "3" );
	       }
	       if ( (param.chan_mask & BAND_FOUR) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "4" );
	       }
	       if ( (param.chan_mask & BAND_FIVE) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "5" );
	       }
	       if ( (param.chan_mask & BAND_SIX) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "6" );
	       }
	       if ( (param.chan_mask & BAND_SEVEN) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "7" );
	       }
	       if ( (param.chan_mask & BAND_EIGHT) != UCHAR_ZERO ) {
		    if ( strlen( string ) > 0 ) (void) strcat( string, "," );
		    (void) strcat( string, "8" );
	       }
	       if ( strlen( string ) == 0 )
		    nadc_write_text( outfl, ++nr, "Bands", "None" );
	       else
		    nadc_write_text( outfl, ++nr, "Bands", string );
	  }
/*
 * selected Clusters
 */
	  if ( param.clusID_nr == PARAM_UNSET ) {
	       nadc_write_text( outfl, ++nr, "Clusters", "All" );
	  } else {
	       *cbuff = '\0';
	       for ( ii = 0; ii < MAX_NUM_CLUS; ii++ ) {
		    if (Get_Bit_LL(param.clus_mask,(unsigned char)ii) == 0ULL)
			 (void) strcat( cbuff, "0" );
		    else
			 (void) strcat( cbuff, "1" );
	       }
	       nadc_write_text( outfl, ++nr, "Clusters", cbuff );
	  }
/*
 * calibration of the spectral band data
 */
	  if ( param.calib_scia == CALIB_NONE ) {
	       nadc_write_text( outfl, ++nr, "Calibration", "None" );
	  } else {
	       adim = NADC_SCIA_CalibStr( param.calib_scia, string );
	       if ( adim == 0 )
		    nadc_write_text( outfl, ++nr, "Calibration", "None" );
	       else
		    nadc_write_text( outfl, ++nr, "Calibration", string );
	  }
	  break;
/*
 *  ----- SCIAMACHY level 2 processor specific options
 */
     case SCIA_LEVEL_2:
/*
 * selected Key DSD's
 */
	  (void) strcpy( cbuff, "" );
	  if ( param.write_ads == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoADS" );
	  }
	  if ( param.write_gads == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoGADS" );
	  }
	  if ( strlen( cbuff ) == 0 ) (void) strcpy( cbuff, "All" );
	  nadc_write_text( outfl, ++nr, "KeyDataSets", cbuff );
/*
 * selected measurement data sets
 */
	  (void) strcpy( cbuff, "" );
	  if ( param.write_cld == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoCLD" );
	  }
/* NRT, only */
	  if ( param.write_bias == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoBIAS" );
	  }
	  if ( param.write_doas == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoDOAS" );
	  }
/* offline, only */
	  if ( param.write_limb == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoLimb" );
	  }
	  if ( param.write_nadir == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoNadir" );
	  }
	  if ( param.write_occ == PARAM_UNSET ) {
	       if ( strlen( cbuff ) > 0 ) (void) strcat( cbuff, "," );
	       (void) strcat( cbuff, "NoOcc" );
	  }
	  if ( strlen( cbuff ) == 0 ) (void) strcpy( cbuff, "All" );
	  nadc_write_text( outfl, ++nr, "MeasurementDataSets", cbuff );
	  break;
     }
}
