/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2007 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   NADC_CONNECT_DB
.AUTHOR      R.M. van Hees
.KEYWORDS    NADC PostgreSQL
.LANGUAGE    ANSI C
.PURPOSE     connect to meta/pixel database (reading XML configuration file)
.INPUT/OUTPUT
  call as   NADC_CONNECT_DB( conn, database );
     input:  
             PGconn *conn   :  PostgreSQL connection handle
	     char *database :  name of database to be used

.RETURNS     Nothing, error status passed by global variable ``nadc_stat''
.COMMENTS    None
.ENVIRONment None
.VERSION     2.0     08-Jun-2007   port to PostgreSQL by R. M. van Hees
             1.0     05-Jun-2007   initial release by R. M. van Hees
------------------------------------------------------------*/
/*
 * Define _ISOC99_SOURCE to indicate
 * that this is a ISO C99 program
 */
#define  _ISOC99_SOURCE

/*+++++ System headers +++++*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <libpq-fe.h>

/*+++++ Local Headers +++++*/
#include <defs_nadc.h>

/*+++++ Macros +++++*/
	/* NONE */

/*+++++ Global Variables +++++*/
	/* NONE */

/*+++++ Static Variables +++++*/
	/* NONE */

/*+++++++++++++++++++++++++ Static Functions +++++++++++++++++++++++*/
				/* NONE */

/*+++++++++++++++++++++++++ Main Program or Function +++++++++++++++*/
void NADC_CONNECT_DB( PGconn **conn, const char *database )
{
     const char prognm[] = "NADC_CONNECT_DB";

     char conninfo[MAX_STRING_LENGTH];

     char xml_db_host[SHORT_STRING_LENGTH];
     char xml_db_user[SHORT_STRING_LENGTH];
     char xml_db_pass[SHORT_STRING_LENGTH];
     char xml_db_port[SHORT_STRING_LENGTH];

     char *user_home = getenv( "HOME" );
     char *db_host = getenv( "NADC_DB_HOST" );
     char *db_user = getenv( "NADC_DB_USER" );
     char *db_pass = getenv( "NADC_DB_PASSWD" );
     char *db_port = getenv( "NADC_DB_PORT" );
/*
 * obtain host, user and password from NADC XML configuration file
 */
     *conn = NULL;
     if ( db_host == NULL || db_user == NULL ) {
	  FILE *fp = NULL;

	  char config_file[MAX_STRING_LENGTH];

	  (void) strcpy( xml_db_host, "" );
	  (void) strcpy( xml_db_user, "" );
	  (void) strcpy( xml_db_pass, "" );
	  (void) strcpy( xml_db_port, "" );

	  (void) strcpy( config_file, "./nadc.config.xml" );
	  if ( (fp = fopen( config_file, "r" )) == NULL ) {
	       (void) snprintf( config_file, MAX_STRING_LENGTH,
				"%s/nadc.config.xml", user_home );
	       if ( (fp = fopen( config_file, "r" )) == NULL )
		    NADC_RETURN_ERROR( prognm, NADC_ERR_FILE, config_file );
	  }
	  NADC_GET_XML_METADB( fp, xml_db_host, xml_db_port, 
			       xml_db_user, xml_db_pass );
	  if ( db_host == NULL && strlen(xml_db_host) > 0 ) 
	       db_host = xml_db_host;
	  if ( db_user == NULL && strlen(xml_db_user) > 0) 
	       db_user = xml_db_user;
	  if ( db_pass == NULL && strlen(xml_db_pass) > 0) 
	       db_pass = xml_db_pass;
	  if ( db_port == NULL && strlen(xml_db_port) > 0) 
	       db_port = xml_db_port;

	  (void) fclose( fp);
     }
/*
 * make connection to the database
 */
     if ( db_pass == NULL && db_port == NULL )
	  (void) snprintf( conninfo, MAX_STRING_LENGTH,
			   "host=%s user=%s dbname=%s", 
			   db_host, db_user, database );
     else if ( db_port == NULL )
	  (void) snprintf( conninfo, MAX_STRING_LENGTH,
			   "host=%s user=%s password=%s dbname=%s", 
			   db_host, db_user, db_pass, database );
     else if ( db_pass == NULL )
	  (void) snprintf( conninfo, MAX_STRING_LENGTH,
			   "host=%s port=%s user=%s dbname=%s", 
			   db_host, db_port, db_user, database );
     else
	  (void) snprintf( conninfo, MAX_STRING_LENGTH,
			   "host=%s port=%s user=%s password=%s dbname=%s", 
			   db_host, db_port, db_user, db_pass, database );

     *conn = PQconnectdb( conninfo );
     if ( PQstatus( *conn ) != CONNECTION_OK ) 
	  NADC_RETURN_ERROR( prognm, NADC_ERR_SQL, PQerrorMessage(*conn) );
}
