/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.COPYRIGHT (c) 2009 - 2013 SRON (R.M.van.Hees@sron.nl)

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License, version 2, as
   published by the Free Software Foundation.

   The software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, 
   Boston, MA  02111-1307, USA.

.IDENTifer   NADC_SCIA_L1B_NAME
.AUTHOR      R.M. van Hees
.KEYWORDS    SCIA SQL
.LANGUAGE    ANSI C
.PURPOSE     obtain Sciamachy level 1b product name from database
.INPUT/OUTPUT
  call as   NADC_SCIA_L1B_NAME( absOrbit, fileCounter, l1b_product );

     input:  
             unsigned int absOrbit      :
             unsigned short fileCounter :
    output:  
             char *l1b_product          :

.RETURNS     Nothing
.COMMENTS    None
.ENVIRONment None
.VERSION     1.1     08-Feb-2010   Select highest procStage
                                   alternatively, ignore fileCounter, RvH
             1.0     09-Oct-2009   initial release by R. M. van Hees
------------------------------------------------------------*/
/*
 * Define _ISOC99_SOURCE to indicate
 * that this is a ISO C99 program
 */
#define  _ISOC99_SOURCE

/*+++++ System headers +++++*/
#include <stdlib.h>
#include <string.h>

#include <libpq-fe.h>

/*+++++ Local Headers +++++*/
#define _SCIA_COMMON
#include <defs_nadc.h>

/*+++++ Macros +++++*/
#define SQL_STR_SIZE   128

#define SQL_GET_L1B_NAME \
"SELECT name FROM meta__1P WHERE name LIKE '%%_%05u_%04hu.N1' order by procStage DESC"

#define SQL_GET_L1B_NAME2 \
"SELECT name FROM meta__1P WHERE name ~ 'SCI_NL__1P[RU].{37}_%05u_.{4}.N1'"

/*+++++ Global Variables +++++*/
	/* NONE */

/*+++++ Static Variables +++++*/
	/* NONE */

/*+++++++++++++++++++++++++ Static Functions +++++++++++++++++++++++*/
				/* NONE */

/*+++++++++++++++++++++++++ Main Program or Function +++++++++++++++*/
void NADC_SCIA_L1B_NAME( unsigned int absOrbit, unsigned short fileCounter,
			 /*@out@*/ char *l1b_product )
{
     const char prognm[] = "NADC_SCIA_L1B_NAME";

     PGconn *conn;
     PGresult *res;

     int   nrow;

     char  sql_query[SQL_STR_SIZE];

     (void) strcpy( l1b_product, "unknown" );
/*
 * connect to PostgreSQL database
 */
     NADC_CONNECT_DB( &conn, "scia" );
     if ( IS_ERR_STAT_FATAL )
	  NADC_RETURN_ERROR( prognm, NADC_ERR_SQL, "failed to connect" );

     (void) snprintf( sql_query, SQL_STR_SIZE, SQL_GET_L1B_NAME,
		      absOrbit, fileCounter );
     res = PQexec( conn, sql_query );
     if ( PQresultStatus( res ) != PGRES_TUPLES_OK ) {
	  NADC_GOTO_ERROR( prognm, NADC_ERR_SQL, PQresultErrorMessage(res) );
     }
     if ( (nrow = PQntuples( res )) != 0 ) {
	  (void) strlcpy( l1b_product, PQgetvalue(res,0,0), 
			  ENVI_FILENAME_SIZE );
     } else {
	  PQclear( res );
	  (void) snprintf( sql_query, SQL_STR_SIZE, 
			   SQL_GET_L1B_NAME2, absOrbit );
	  res = PQexec( conn, sql_query );
	  if ( PQresultStatus( res ) != PGRES_TUPLES_OK )
	       NADC_GOTO_ERROR(prognm, NADC_ERR_SQL, PQresultErrorMessage(res));

	  if ( (nrow = PQntuples( res )) != 0 )
	       (void) strlcpy( l1b_product, PQgetvalue(res,0,0), 
			       ENVI_FILENAME_SIZE );
     }
 done:
     PQclear( res );
     PQfinish( conn );
}
