DROP TABLE IF EXISTS imap_tileinfo;
DROP TABLE IF EXISTS imap_ch4_tileinfo;
DROP TABLE IF EXISTS imap_hdo_tileinfo;
DROP TABLE IF EXISTS imlm_tileinfo;
DROP TABLE IF EXISTS imlm_co_tileinfo;
DROP TABLE IF EXISTS imlm_h2o_tileinfo;

DROP TABLE IF EXISTS tile_imap;
DROP TABLE IF EXISTS tile_imap_ch4;
DROP TABLE IF EXISTS tile_imap_hdo;
DROP TABLE IF EXISTS tile_imlm;
DROP TABLE IF EXISTS tile_imlm_co;
DROP TABLE IF EXISTS tile_imlm_h2o;

DROP TABLE IF EXISTS meta_imap;
DROP TABLE IF EXISTS meta_imap_ch4;
DROP TABLE IF EXISTS meta_imap_hdo;
DROP TABLE IF EXISTS meta_imlm;
DROP TABLE IF EXISTS meta_imlm_co;
DROP TABLE IF EXISTS meta_imlm_h2o;

CREATE TABLE meta_imap_ch4 (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets      smallint    NOT NULL
);
CREATE TABLE meta_imap_hdo (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  numDataSets      smallint    NOT NULL
);
CREATE TABLE meta_imlm_co (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  pixelMaskVersion varchar(16) NOT NULL,
  cloudMaskVersion varchar(16) NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  windowPixel      integer[2]  NOT NULL,
  windowWave       real[2]     NOT NULL,
  numDataSets      smallint    NOT NULL
);
CREATE TABLE meta_imlm_h2o (
  pk_meta          serial      PRIMARY KEY,
  name             varchar(66) NOT NULL UNIQUE CHECK (name <> ''),
  fk_name_l1b      char(62)    REFERENCES meta__1P(name),
  fileSize         integer     NOT NULL CHECK (fileSize > 0),
  receiveDate      timestamp   NOT NULL,
  creationDate     timestamp   NOT NULL,
  dateTimeStart    timestamp   CHECK (numDataSets > 0 OR dateTimeStart IS NULL),
  muSecStart       integer     CHECK (numDataSets > 0 AND muSecStart >= 0 OR muSecStart IS NULL),
  dateTimeStop     timestamp   CHECK (numDataSets > 0 OR dateTimeStop IS NULL),
  muSecStop        integer     CHECK (numDataSets > 0 AND muSecStop >= 0 OR muSecStop IS NULL),
  softVersion      varchar(8)  NOT NULL,
  pixelMaskVersion varchar(16) NOT NULL,
  cloudMaskVersion varchar(16) NOT NULL,
  absOrbit         integer     NOT NULL CHECK (absOrbit >= 0),
  windowPixel      integer[2]  NOT NULL,
  windowWave       real[2]     NOT NULL,
  numDataSets      smallint    NOT NULL
);

CREATE TABLE tile_imap_ch4 (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imap_ch4(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  VCD_CH4          real      NOT NULL,
  VCD_CH4_ERROR    real      NOT NULL,
  VCD_CO2          real      NOT NULL,
  VCD_CO2_ERROR    real      NOT NULL,
  xVMR_CH4         real      NOT NULL
);
CREATE TABLE tile_imap_hdo (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imap_hdo(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  VCD_HDO          real      NOT NULL,
  VCD_HDO_ERROR    real      NOT NULL,
  VCD_H2O          real      NOT NULL,
  VCD_H2O_ERROR    real      NOT NULL,
  VCD_H2O_MODEL    real      NOT NULL
);
CREATE TABLE tile_imlm_co (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imlm_co(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  errorFlag        smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  cloudFraction    real      ,
  surfaceAlbedo    real      ,
  VCD_CO           real      ,
  VCD_CO_ERROR     real      ,
  VCD_CH4          real      ,
  VCD_CH4_ERROR    real      
);
CREATE TABLE tile_imlm_h2o (
  pk_tile          bigserial PRIMARY KEY,
  julianDay        double precision NOT NULL UNIQUE,
  fk_meta          integer   REFERENCES meta_imlm_h2o(pk_meta) ON DELETE CASCADE,
  integrationTime  smallint  NOT NULL,
  errorFlag        smallint  NOT NULL,
  meanElevation    real      NOT NULL,
  cloudFraction    real      ,
  surfaceAlbedo    real      ,
  VCD_H2O          real      ,
  VCD_H2O_ERROR    real      ,
  VCD_CH4          real      ,
  VCD_CH4_ERROR    real      
);

CREATE TABLE imlm_co_tileinfo (
  fk_tile       bigint   REFERENCES tile_imlm_co(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

CREATE TABLE imlm_h2o_tileinfo (
  fk_tile       bigint   REFERENCES tile_imlm_h2o(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);
CREATE TABLE imap_ch4_tileinfo (
  fk_tile       bigint   REFERENCES tile_imap_ch4(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);
CREATE TABLE imap_hdo_tileinfo (
  fk_tile       bigint   REFERENCES tile_imap_hdo(pk_tile) ON DELETE CASCADE,
  fk_tileinfo   bigint   REFERENCES tileinfo(pk_tileinfo),
  PRIMARY KEY (fk_tile, fk_tileinfo)
);

GRANT SELECT ON imap_ch4_tileinfo, imap_hdo_tileinfo, imlm_co_tileinfo, imlm_h2o_tileinfo, meta_imap_ch4, meta_imap_hdo, meta_imap_ch4_pk_meta_seq, meta_imap_hdo_pk_meta_seq, tile_imap_ch4, tile_imap_hdo, tile_imap_ch4_pk_tile_seq, tile_imap_hdo_pk_tile_seq, tile_imlm_co, tile_imlm_h2o, tile_imlm_co_pk_tile_seq, tile_imlm_h2o_pk_tile_seq TO nadc_user;
GRANT USAGE ON SCHEMA nadc_admin to nadc_user;
